#include <optional>

#include <emulator/State.hpp>

struct StateView {
  LengthUnit speed;
};

class Controller {
public:
  explicit Controller(const State &state);

  void modify_state(State &state);

  static constexpr LengthUnit default_deceleration_rate{0.01};
  static constexpr ForceUnit default_gas_pedal_force{0.5};
  static constexpr ForceUnit default_break_pedal_force{0.2};

private:
  LengthUnit get_deceleration_rate() const;
  double compute_breaking_time(LengthUnit speed) const;
  LengthUnit compute_breaking_distance(LengthUnit speed) const;

  std::optional<TrafficSign> get_last_sign(LengthUnit position) const;

  void update_deceleration_rate(LengthUnit speed);

private:
  const LengthUnit max_speed;
  const LengthUnit min_speed;
  const Location starting_location;
  const Location destination;
  const std::vector<TrafficSign> signs;

  ForceUnit gas_pedal_force;
  ForceUnit break_pedal_force;

  std::optional<StateView> previous_state;
  std::optional<LengthUnit> deceleration_rate;
};