#include <optional>
#include <vector>

#include <emulator/Direction.hpp>
#include <emulator/Lane.hpp>
#include <emulator/Location.hpp>
#include <emulator/Road.hpp>

enum class Symbol : char {
  EMPTY = ' ',
  LANE_BORDER = '.',
  MY_CAR = '@',
  OTHER_CAR = 'o',
  DESTINATION = 'X',
  SIGN = 'T',
};

struct LaneView {
  int x_min, x_max, y_min, y_max;
};

struct Map {
  Map();

  std::vector<std::vector<Symbol>> buffer;
  LengthUnit x;
  LengthUnit y;
  LengthUnit speed;

  static constexpr size_t map_radius = 15;
  static constexpr size_t map_size = 2 * map_radius + 1;
};

class Formatter {
public:
  explicit Formatter(const Location &loc);

  void format_lanes(const std::vector<LaneView> &lanes);
  void format_my_car();
  void format_other_cars(const std::vector<Location> &locations);
  void format_my_params(const Location &location, LengthUnit speed);
  void format_signs(const std::vector<LengthUnit> &positions);
  void format_destination(const Location &location);

  void print(bool draw_map = true) const;

private:
  int row_to_y(size_t row) const {
    return Map::map_size - 1 - static_cast<int>(row) + this->origin_y;
  }
  int col_to_x(size_t col) const {
    return static_cast<int>(col) + this->origin_x;
  }

  std::optional<size_t> y_to_row(int y) const {
    auto r = static_cast<int>(Map::map_radius);
    if ((y < this->origin_y) || (y > this->origin_y + 2 * r)) {
      return std::nullopt;
    }
    return static_cast<size_t>(Map::map_size - 1 - y + this->origin_y);
  }

  std::optional<size_t> x_to_col(int x) const {
    auto r = static_cast<int>(Map::map_radius);
    if ((x < this->origin_x) || (x > this->origin_x + 2 * r)) {
      return std::nullopt;
    }
    return static_cast<size_t>(x - this->origin_x);
  }

private:
  const int origin_x;
  const int origin_y;

private:
  Map map;
};