#include <algorithm>
#include <iostream>

#include <controller/Formatter.h>

Map::Map() : buffer(map_size, std::vector<Symbol>(map_size, Symbol::EMPTY)){};

Formatter::Formatter(const Location &loc)
    : origin_x{static_cast<int>(std::round(loc.x)) -
               static_cast<int>(Map::map_radius)},
      origin_y{static_cast<int>(std::round(loc.y)) -
               static_cast<int>(Map::map_radius)},
      map(){};

void Formatter::format_lanes(const std::vector<LaneView> &lanes) {
  auto is_any_lane_border = [&lanes](int x, int y) {
    auto is_lane_border = [&x, &y](const LaneView &lane) {
      int x_min = lane.x_min;
      int x_max = lane.x_max;
      int y_min = lane.y_min;
      int y_max = lane.y_max;
      return ((x == x_min) && (y >= y_min) && (y <= y_max)) ||
             ((x == x_max) && (y >= y_min) && (y <= y_max)) ||
             ((y == y_min) && (x >= x_min) && (x <= x_max)) ||
             ((y == y_max) && (x >= x_min) && (x <= x_max));
    };
    return std::any_of(lanes.begin(), lanes.end(), is_lane_border);
  };

  for (size_t row = 0; row < Map::map_size; ++row) {
    auto &map_row = this->map.buffer[row];
    int y = this->row_to_y(row);
    for (size_t col = 0; col < Map::map_size; ++col) {
      int x = this->col_to_x(col);
      if (is_any_lane_border(x, y)) {
        this->map.buffer[row][col] = Symbol::LANE_BORDER;
      }
    }
  }
}

void Formatter::format_my_car() {
  this->map.buffer[Map::map_radius][Map::map_radius] = Symbol::MY_CAR;
}

void Formatter::format_other_cars(const std::vector<Location> &locations) {
  for (const Location &loc : locations) {
    auto col = this->x_to_col(static_cast<int>(std::round(loc.x)));
    auto row = this->y_to_row(static_cast<int>(std::round(loc.y)));
    if (!row.has_value() || !col.has_value()) {
      continue;
    }
    this->map.buffer[*row][*col] = Symbol::OTHER_CAR;
  }
}

void Formatter::format_my_params(const Location &location, LengthUnit speed) {
  this->map.x = location.x;
  this->map.y = location.y;
  this->map.speed = speed;
}

void Formatter::format_signs(const std::vector<LengthUnit> &positions) {
  for (const LengthUnit x : positions) {
    auto col = this->x_to_col(static_cast<int>(std::round(x)));
    if (!col.has_value()) {
      continue;
    }
    this->map.buffer[Map::map_size - 1][*col] = Symbol::SIGN;
  }
}

void Formatter::format_destination(const Location &location) {
  auto col = this->x_to_col(static_cast<int>(std::round(location.x)));
  auto row = this->y_to_row(static_cast<int>(std::round(location.y)));
  if (row.has_value() && col.has_value()) {
    this->map.buffer[*row][*col] = Symbol::DESTINATION;
  }
}

void Formatter::print(bool draw_map) const {
  if (draw_map) {
    size_t idx = 0;
    char buffer[(Map::map_size + 1) * (Map::map_size) + 1];
    for (size_t row = 0; row < Map::map_size; ++row) {
      for (size_t col = 0; col < Map::map_size; ++col) {
        buffer[idx] = static_cast<char>(this->map.buffer[row][col]);
        ++idx;
      }
      buffer[idx] = '\n';
      ++idx;
    }
    buffer[idx] = '\0';
    std::cout << buffer;
  }
  std::cout << "x=" << this->map.x << ", y=" << this->map.y
            << ", v=" << this->map.speed << std::endl;
}